from blueness import module

from blue_objects import mlflow, metadata

from blue_sandbox import NAME
from blue_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def ingest(
    event_name: str,
    object_name: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.ingest({event_name}) -> {object_name}")

    return all(
        [
            mlflow.set_tags(
                object_name,
                {"event": event_name},
            ),
            metadata.post_to_object(
                object_name,
                "ingest",
                {"event": event_name},
            ),
        ]
    )



import os

from blue_objects import file, README

from blue_sandbox import NAME, VERSION, ICON, REPO_NAME
from blue_sandbox.microsoft_building_damage_assessment import (
    README as microsoft_building_damage_assessment_README,
)
from blue_sandbox.list import list_of_experiments

items = [
    "{}[`{}`]({}) {} [![image]({})]({}) {}".format(
        experiment["ICON"],
        experiment_name,
        experiment["url"],
        experiment["status"],
        experiment["marquee"],
        experiment["url"],
        experiment["title"],
    )
    for experiment_name, experiment in list_of_experiments.items()
    if experiment_name != "template"
]


def build():
    return all(
        [
            README.build(
                items=items,
                path=os.path.join(file.path(__file__), ".."),
                ICON=ICON,
                NAME=NAME,
                VERSION=VERSION,
                REPO_NAME=REPO_NAME,
            ),
            README.build(
                items=microsoft_building_damage_assessment_README.items,
                cols=len(microsoft_building_damage_assessment_README.list_of_steps),
                path=os.path.join(
                    file.path(__file__), "microsoft_building_damage_assessment"
                ),
                ICON=ICON,
                NAME=NAME,
                VERSION=VERSION,
                REPO_NAME=REPO_NAME,
            ),
        ]
        + [
            README.build(
                path=os.path.join(file.path(__file__), experiment_name),
                ICON=ICON,
                NAME=NAME,
                VERSION=VERSION,
                REPO_NAME=REPO_NAME,
            )
            for experiment_name in ["cemetery", "palisades", "sagesemseg"]
        ]
    )



import os
from blue_options.env import load_config, load_env

load_env(__name__)
load_config(__name__)


DAMAGES_TEST_DATASET_OBJECT_NAME = os.getenv(
    "DAMAGES_TEST_DATASET_OBJECT_NAME",
    "",
)

ENCODED_BLOB_SAS_TOKEN = os.getenv("ENCODED_BLOB_SAS_TOKEN", "")

SAGESEMSEG_COMPLETED_JOB_pascal_voc_v1_debug_v2 = os.getenv(
    "SAGESEMSEG_COMPLETED_JOB_pascal_voc_v1_debug_v2",
    "",
)

SAGESEMSEG_COMPLETED_JOB_pascal_voc_v1_full_v2 = os.getenv(
    "SAGESEMSEG_COMPLETED_JOB_pascal_voc_v1_full_v2",
    "",
)



from typing import Dict

from blue_sandbox.env import DAMAGES_TEST_DATASET_OBJECT_NAME

list_of_steps: Dict[str, Dict] = {
    "ingest": {
        "object_name": DAMAGES_TEST_DATASET_OBJECT_NAME,
        "image_name": "Maui-Hawaii-fires-Aug-23-damage-2025-01-09-GgnjQC",
    },
    "label": {
        "object_name": "",
        "image_name": DAMAGES_TEST_DATASET_OBJECT_NAME,
    },
    "train": {
        "object_name": "",
        "image_name": "Maui-Hawaii-fires-Aug-23-model-2025-01-10-NQb8IS",
    },
    "predict": {"object_name": "", "image_name": ""},
    "summarize": {"object_name": "", "image_name": ""},
}

items = (
    [f"`{step}`" for step in list_of_steps]
    + [
        (
            "[`{}`](https://kamangir-public.s3.ca-central-1.amazonaws.com/{}.tar.gz)".format(
                step["object_name"],
                step["image_name"],
            )
            if step["object_name"]
            else ""
        )
        for step in list_of_steps.values()
    ]
    + [
        (
            "![image](https://github.com/kamangir/assets/blob/main/blue-sandbox/{}.png?raw=true)".format(
                step["image_name"],
            )
            if step["image_name"]
            else ""
        )
        for step in list_of_steps.values()
    ]
)



NAME = "blue_sandbox"

ICON = "🌀"

DESCRIPTION = f"{ICON} A sandbox for ideas and experiments."

VERSION = "5.148.1"

REPO_NAME = "blue-sandbox"

MARQUEE = (
    "https://github.com/kamangir/assets/raw/main/blue-sandbox/marquee.png?raw=true"
)

ALIAS = "@sandbox"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



list_of_experiments = {
    "palisades": {
        "ICON": "🧑🏽‍🚒",
        "title": "Post-disaster Land Cover Classification.",
        "url": "https://github.com/kamangir/blue-sandbox/blob/main/blue_sandbox/palisades/README.md",
        "marquee": "https://github.com/kamangir/assets/raw/main/blue-geo/Maxar-Open-Datacube.png?raw=true",
        "status": "🔥",
    },
    "sagesemseg": {
        "ICON": "🌀",
        "title": "A SemSeg (Semantic Segmenter) trained and deployed on AWS Sagemaker.",
        "url": "https://github.com/kamangir/blue-sandbox/blob/main/blue_sandbox/sagesemseg/README.md",
        "marquee": "https://github.com/kamangir/assets/blob/main/blue-sandbox/sagesemseg-predict.png?raw=true",
        "status": "⏸️",
    },
    "`@damages`": {
        "ICON": "🌐",
        "title": "Satellite imagery damage assessment workflow",
        "url": "https://github.com/kamangir/blue-sandbox/blob/main/blue_sandbox/microsoft_building_damage_assessment/README.md",
        "marquee": "https://github.com/kamangir/assets/raw/main/blue-sandbox/Maui-Hawaii-fires-Aug-23-ingest-2025-01-10-qqJqhm.png?raw=true",
        "status": "⏸️",
    },
    "cemetery": {
        "ICON": "🪦",
        "title": "An AI cemetery.",
        "url": "https://github.com/kamangir/blue-sandbox/blob/main/blue_sandbox/cemetery/README.md",
        "marquee": "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true",
        "status": "🪦",
    },
    "template": {
        "ICON": "",
        "title": "",
        "url": "",
        "marquee": "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true",
        "status": "🎰",
    },
}



import os

from blueness import module
from blue_objects import metadata, file, objects
from blue_objects.env import abcli_path_git

from blue_sandbox import NAME
from blue_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def label(
    object_name: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.label: {object_name}")

    geojson_filename = os.path.join(
        abcli_path_git,
        "building-damage-assessment/data/demo/labels/Maui_Wildfires_August_0.geojson",
    )

    if not file.copy(
        geojson_filename,
        objects.path_of(
            filename="label.geojson",
            object_name=object_name,
        ),
    ):
        return False

    return metadata.post_to_object(
        object_name,
        "label",
        {"geojson": geojson_filename},
    )



import base64


def encode_token(token: str) -> str:
    return base64.b64encode(token.encode()).decode()


def decode_token(encoded_token: str) -> str:
    return base64.b64decode(encoded_token.encode()).decode()



import os

from blueness import module
from blue_objects import mlflow, metadata, file, objects, path
from blue_objects.env import abcli_path_git

from blue_sandbox import NAME
from blue_sandbox.env import ENCODED_BLOB_SAS_TOKEN
from blue_sandbox.microsoft_building_damage_assessment.sas_token import decode_token
from blue_sandbox.logger import logger


NAME = module.name(__file__, NAME)


# ... copy `configs/example_config.yml` and fill the first three sections.
def train(
    dataset_object_name: str,
    model_object_name: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.train: {dataset_object_name} -> {model_object_name}")

    config_filename = os.path.join(
        abcli_path_git,
        "building-damage-assessment/configs/example_config.yml",
    )
    success, config = file.load_yaml(config_filename)
    if not success:
        return False

    config["experiment_dir"] = objects.object_path(model_object_name)
    config["experiment_name"] = model_object_name

    config["imagery"]["rgb_fn"] = config["imagery"]["raw_fn"] = objects.path_of(
        filename="raw/maxar_lahaina_8_12_2023-visual.tif",
        object_name=dataset_object_name,
    )

    config["inference"]["output_subdir"] = objects.path_of(
        filename="outputs/",
        object_name=model_object_name,
    )
    config["inference"]["checkpoint_fn"] = objects.path_of(
        filename="checkpoints/last.ckpt",
        object_name=model_object_name,
    )

    config["infrastructure"]["container_name"] = "sandbox"
    config["infrastructure"][
        "storage_account"
    ] = "https://kamangir.blob.core.windows.net/"
    config["infrastructure"]["sas_token"] = decode_token(ENCODED_BLOB_SAS_TOKEN)
    config["infrastructure"]["relative_path"] = dataset_object_name

    config["labels"]["fn"] = objects.path_of(
        filename="label.geojson",
        object_name=dataset_object_name,
    )

    config["training"]["log_dir"] = objects.path_of(
        filename="logs/",
        object_name=model_object_name,
    )
    config["training"]["checkpoint_subdir"] = objects.path_of(
        filename="checkpoints/",
        object_name=model_object_name,
    )

    if not file.save_yaml(
        objects.path_of(
            filename="config.yml",
            object_name=model_object_name,
        ),
        config,
    ):
        return False

    return all(
        [
            mlflow.set_tags(
                model_object_name,
                {
                    "dataset": dataset_object_name,
                },
            ),
            metadata.post_to_object(
                model_object_name,
                "train",
                {
                    "dataset": dataset_object_name,
                },
            ),
        ]
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_sandbox import NAME
from blue_sandbox.microsoft_building_damage_assessment.ingest import ingest
from blue_sandbox.microsoft_building_damage_assessment.label import label
from blue_sandbox.microsoft_building_damage_assessment.train import train
from blue_sandbox.microsoft_building_damage_assessment.sas_token import (
    decode_token,
    encode_token,
)
from blue_sandbox.help.microsoft_building_damage_assessment import list_of_events
from blue_sandbox.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="decode_token | encode_token | ingest | label | train",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--dataset_object_name",
    type=str,
)
parser.add_argument(
    "--model_object_name",
    type=str,
)
parser.add_argument(
    "--event_name",
    type=str,
    default=list_of_events[0],
    help=" | ".join(list_of_events),
)
parser.add_argument(
    "--verbose",
    type=bool,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--token",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "decode_token":
    success = True
    print(decode_token(args.token))
elif args.task == "encode_token":
    success = True
    print(encode_token(args.token))
elif args.task == "ingest":
    success = ingest(
        object_name=args.object_name,
        event_name=args.event_name,
        verbose=args.verbose == 1,
    )
elif args.task == "label":
    success = label(
        object_name=args.object_name,
        verbose=args.verbose == 1,
    )
elif args.task == "train":
    success = train(
        dataset_object_name=args.dataset_object_name,
        model_object_name=args.model_object_name,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)
